/*
   Widgetklasse:	XmGrFormClass
   GrForm.h

$Revision: 1.1 $

$Log: GrForm.h,v $
Revision 1.1  2008/12/23 14:22:33  keidel
added used parts of easymotif lib as source

Revision 1.3  2004/06/05 07:46:59  keidel
bug fixes

Revision 4.0  2004/02/28 09:53:00  keidel
Major Version

Revision 1.2  2004/02/28 09:20:23  keidel
complete graphic-extension, creating C-code, save/restore, display and editing
Line, Polygon, Circle, Rectangle
moving points, setting effects, color, style, fill, linewidth etc.

inhibit buttonpress while pointer grabbed.
corrected GrForm for change managed
attachments to Form with Frame/Scroll corrected
better reaction on -UGRFORM
use Grid of GrForm if GrForm enabled  (GrGrid added to GrForm)

Revision 1.1  2004/02/23 14:56:04  keidel
added GrForm widget as extewrnsion to XmForm. This adds
a graphic extension to the standard form, including dynamic effects
and grid.

Revision 1.1  2003/12/29 10:15:23  keidel
add histspectr widget
add GRForm widget, replace XmForm conditionally

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2, or (at your option)
        any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <Xm/Form.h>
#include <stdarg.h>

#ifndef _XmGrForm_h
#define _XmGrForm_h


#ifdef __cplusplus
extern "C"{
#endif

#define GrNLineWidth    "LineWidth"
#define GrNColor        "Color"       /* Pixel */
#define GrNLineStyle    "LineStyle"
#define GrNPositionTyp  "PositionTyp"    /* (enum) GR_RELATIVE or GR_ABSOLUTE */
#define GrNVisible      "Visible"     /* Boolean */
#define GrNObject       "Object"
#define GrNPrimitive    "Primitive"
#define GrNVertex       "Vertex"
#define GrNArrow        "Arrow"
#define GrNArrowSize    "ArrowSize"
#define GrNEffect       "Effect"                /* Object only */
#define GrNFlowIncrement "FlowIncrement"        /* Object only */
#define GrNDashes        "Dashes"               /* Object only */
#define GrNAttachTyp     "AttachTyp"
#define GrNAttachWidgetX  "AttachWidgetX"
#define GrNAttachWidgetY  "AttachWidgetY"
#define GrNx              "Vx"
#define GrNy              "Vy"
#define GrNevent          "GrEvent"
#define GrCEvent          "grevent"
#define GrEvent           "Gre"


enum primtypes {BAD=0,GR_LINES,GR_LINE,GR_RECTANGLE,GR_POLYGON,GR_CIRCLE,GR_GRID};
enum arrowpos  {Noarr=0 , GRBEGIN,GREND};
enum effects   {NOEFFECT,GRFLOW,GRBLINK};
enum Attaches{GR_RELATIVE,GR_ABSOLUTE,GR_X_RIGHT=2,GR_X_MIDDLE=4,GR_X_LEFT=6,GR_Y_TOP=8,GR_Y_MIDDLE=16,GR_Y_BOTTOM=24};

typedef struct _GrVertex{
/* public area: */
    int       x;          /* in percent of form width */
    int       y;          /* in percent of form height */
    int       flag;       /* defines if absolute, attached (or Form relative as default (==0))
                                or attached to widgets (here x and y are used as offsets) */
    Widget    attached_x;
    Widget    attached_y;
/*              INTERNALS  */    
   Dimension  p_x;        /* calculated coodinate */
   Dimension  p_y;        /* calculated coodinate */         
}GrVertex;

typedef struct _GrPrimitive{
 /*            INTERNALS  */    
    struct _GrPrimitive *next;
    int                  Ident;
    int                 Type;
    int     	        npoints;    /* no of vertices */
    GrVertex            *points;    /* array of Vertex structures */
/*             GLOBALS: */
    Bool                visible;    	    /* overrides the Object defs if set */
    Pixel               color;	    	    /* overrides the Object defs if set */
    int                 linewidth;
    int                 linestyle;
    int                 Switch_condition;   /* overrides the Object defs if set */
    int             	effect;     	    /* overrides the Object defs if set */          
    int     	flag;	    /*  flag == 1   arrow at begin, == 2 at end*/
}GrPrimitive;


    
typedef struct _GrObject{

/*            INTERNALS  */    
    int                 Ident;
    int                 Type;	    	    /* not actually used */
/*             GLOBALS: */
    Pixel               color;              /* stays for all Primitives, if the value there is zero */
    Bool                visible;            /* stays for all Primitives, if the value there is zero */
    int                 linewidth;
    int                 linestyle;
    int                 Switch_condition;   /* stays for all Primitives, if the value there is zero */
    int             	effect;     	    /* special effect stay for all Primit1ves, if possible */          
/*            INTERNALS  */    
    struct _GrObject    *next;
    GrPrimitive         *chain;
    short int           floffset, flincrem, fllimit; 
    char                dashes[4];
}GrObject;


typedef struct {
    XButtonEvent  button;
    char          Mark[4];
    int           objid;
    int           primid;
    int           vertidx;
    GrObject      *obj;
    GrPrimitive   *prim;
    GrVertex      *vert;
}XGrFormEvent;


/*      define primitive structures    */
typedef struct {
    GrPrimitive primitive;
    int nx;
    int ny;
}GrGrid;

typedef struct {
    GrPrimitive primitive;
    int         arrowsize;
}GrLine;    


typedef struct {
    GrPrimitive primitive;
    int         arrowsize;
}GrLines;    

typedef struct {
    GrPrimitive primitive;
}GrRectangle;    
 
typedef struct {
    GrPrimitive primitive;
    int         fill;
}GrPolygon;
 
typedef struct {
    GrPrimitive primitive;
    int    startangle;
    int    rangeangle;
    int    floffset;
}GrCircle;

    
externalref WidgetClass xmGrFormWidgetClass;

enum Primsizes {BAD_S=sizeof(GrPrimitive)
        ,GR_LINE_S=sizeof(GrLine)
        ,GR_LINES_S=sizeof(GrLines)
        ,GR_RECTANGLE_S=sizeof(GrRectangle)
        ,GR_POLYGON_S=sizeof(GrPolygon)
        ,GR_CIRCLE_S=sizeof(GrCircle)
        };


/* Prototypes for public functions   */


int GrForm_New_Object(Widget w);    /* returns a created Objects Id */

                                    /* returns a created Primitive Id */
int GrFormDrawLine(Widget w,int object, int x1, int y1, int x2, int y2, int Mode);
int GrFormDrawLines(Widget w,int object,GrVertex *points, int npoints) ;
int GrFormFillPolygon(Widget w,int object,GrVertex *points, int npoints) ;
int GrFormRectangle(Widget w,int object, int x, int y, int width, int height, int Mode);
int GrFormFillRectangle(Widget w,int object, int x, int y, int width, int height, int Mode);
int GrFormCircle (Widget w,int object,int x_center,int y_center,int radius , int fill , int Mode);
int GrFormGrid (Widget w,int object , int nx , int ny);

        
    /*  change graphic objects */
GrObject *Get_GrForm_Object(Widget w,int obj_id) ;  /* get a created object for changing */
void Set_GrForm_Object(Widget w,GrObject *obj) ;  /* copy parts of changed object over*/
GrPrimitive *Get_GrForm_Primitive(Widget w,int obj_id,int prim_id);  /* get a created primitive for changing */
void Set_GrForm_Primitive(Widget w,int obj,GrPrimitive *prim); /* copy parts of changed primitive over*/

   /* general Functions */
int GrFormDeleteP (Widget w, int objid, int primid );
int GrFormDeleteV (Widget w, int objid, int primid , int vertidx);
int GrFormDeleteO (Widget w, int objid);
 void GrRedraw(Widget w);

void GrForm_Effect(Widget w , int start);

Widget GrCreateForm(Widget parent ,  String name, ArgList argslist, Cardinal argcount );

#ifdef __cplusplus
}
#endif

         
#endif
