/*
   Widgetklasse: 	XmGrFormClass
   GrForm.c

$Revision: 1.1 $

$Log: GrForm.c,v $
Revision 1.1  2008/12/23 14:22:33  keidel
added used parts of easymotif lib as source

Revision 1.4  2004/08/21 07:08:35  keidel
minor bug corrections for LIN/LINES

Revision 4.1  2004/06/05 07:18:55  keidel
PD conditional (if not set) corrected
Menus    loosing focus for selected, deleted pulldown really delete
Wall adaptions

Revision 4.0  2004/02/28 09:53:00  keidel
Major Version

Revision 1.3  2004/02/28 09:20:23  keidel
complete graphic-extension, creating C-code, save/restore, display and editing
Line, Polygon, Circle, Rectangle
moving points, setting effects, color, style, fill, linewidth etc.

inhibit buttonpress while pointer grabbed.
corrected GrForm for change managed
attachments to Form with Frame/Scroll corrected
better reaction on -UGRFORM
use Grid of GrForm if GrForm enabled  (GrGrid added to GrForm)

Revision 1.1  2004/02/23 14:55:58  keidel
added GrForm widget as extewrnsion to XmForm. This adds
a graphic extension to the standard form, including dynamic effects
and grid.

Revision 1.2  2004/01/04 09:36:15  keidel
inhibit buttonpress while pointer grabbed.
corrected GrForm for change managed
attachments to Form with Frame/Scroll corrected

Revision 1.1  2003/12/29 10:15:23  keidel
add histspectr widget
add GRForm widget, replace XmForm conditionally

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2, or (at your option)
        any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/******************************************************************
   Widgetclass:		XmGrFormClass

   written by: 	WizAnt Software
   		Jrgen Keidel  
   		Bergstrasse 15
   		D 85080 Gaimersheim
   		Tel: D  08406 379
   		E-mail: wizant.keidel@freenet.de
   		2003
                                                                  
******************************************************************/

#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include "GrFormP.h"

#ifdef __cplusplus
extern "C"{
#endif

#define offset(field) XtOffsetOf(XmGrFormRec, field)

#ifndef strdup
char *strdup(const char *s);
#endif

#define LEFT	_XmFORM_LEFT
#define RIGHT	_XmFORM_RIGHT
#define TOP	_XmFORM_TOP
#define BOTTOM	_XmFORM_BOTTOM
#define FIRST_ATTACHMENT  LEFT
#define LAST_ATTACHMENT   BOTTOM	

#define GetFormConstraint(w) \
	(&((XmFormConstraintPtr) (w)->core.constraints)->form)
#define SIBLINGS(w,s)	(((w != NULL) && (s != NULL)) &&\
			 (XtParent(w) == XtParent(s)))

#define Calc_X(v) \
              (!( v->flag & (GR_X_RIGHT|GR_X_MIDDLE|GR_X_LEFT)) \
              || (( v->flag & (GR_X_RIGHT|GR_X_MIDDLE|GR_X_LEFT)) \
               && (v->attached_x && !v->attached_x->core.being_destroyed \
               && XtIsManaged(v->attached_x))))
 
#define Calc_Y(v) \
             (!( v->flag & (GR_Y_TOP|GR_Y_MIDDLE|GR_Y_BOTTOM)) \
              || (( v->flag & (GR_Y_TOP|GR_Y_MIDDLE|GR_Y_BOTTOM)) \
               && (v->attached_y && !v->attached_y->core.being_destroyed \
               && XtIsManaged(v->attached_y))))
#define ReDraw(w) if( XtIsRealized(w)){XClearWindow(XtDisplay(w), XtWindow(w)); \
                    redraw((XmGrFormWidget)w );}
                    
/********    Static Function Declarations    ********/
#ifdef _NO_PROTO
static void     Initialize() ;
static void     Resize() ;
static void     Redisplay() ;
static void    ChangeManaged() ;
static void     DestroyfOfChild ( );
static void     Destroy( );

static Boolean  SetValues ( );
static Boolean  SetValuesHook();

static GrObject *Get_Object();
static void     Add_Primitive();
static void     GrDrawObject();
static void     GrRecalculate( );
static void     GrRecalculateVertex();
static int      GrRect();
static int      GrPoly();
static GrPrimitive *Get_Primitive( );
static GrObject *Get_Object();
static void     check_attached ();
static void     ClearAttach ();
static void     arrow ();
static int      ishit ();
static void     redraw ();
static void     draw_grid ();

static void motion_event (); 
#else
static void     Initialize(Widget rw, Widget nw,ArgList args, Cardinal *num_args) ;
static void     Resize(Widget w) ;
static void     Redisplay(Widget wid,XEvent *event, Region region) ;
static void    ChangeManaged( ) ;
static void     DestroyfOfChild (Widget parent, XmGrFormWidget data , XmAnyCallbackStruct * cbs );

static Boolean  SetValues ( Widget curr, Widget requ, Widget new, ArgList args, Cardinal *numargs);
static Boolean  SetValuesHook(Widget ww, ArgList args,Cardinal *num_args);
static void     Destroy(Widget w );

static GrObject *Get_Object(XmGrFormWidget w ,int Id);
static void     Add_Primitive(GrObject *obj , GrPrimitive *p);
static void     GrDrawObject(Widget w , GrObject *obj , int effect);
static void     GrRecalculate(XmGrFormWidget w );
static void     GrRecalculateVertex(XmGrFormWidget w , GrVertex *v);
static int      GrRect(Widget w,int object
                , int x, int y, int width, int height , int flag,int Mode);
static int      GrPoly(Widget w,int object
                ,GrVertex *points, int npoints , int flag );
static GrPrimitive *Get_Primitive( GrObject *obj , int prim_id);
static GrObject *Get_Object(XmGrFormWidget w , int Id);
static void     check_attached (XmGrFormWidget w ,GrVertex *v);
static void     ClearAttach (Widget parent,GrVertex *v);
static void     arrow (Widget w ,GrVertex *beg , GrVertex *end , int length , int flag  );
static int      ishit (Widget w, GrPrimitive *prim , GrVertex **vert , int x , int y);
static void     redraw (XmGrFormWidget cw);
static void     draw_grid (Widget w, GrPrimitive *prim);

static void     motion_event (Widget parent, XtPointer clientData, XEvent *event
                    , Boolean *continueToDispatch); 
#endif /* _NO_PROTO */
 
/********    End Static Function Declarations    ********/

static int primsizes[] = {
        sizeof(GrPrimitive)
        ,sizeof(GrLine)
        ,sizeof(GrLines)
        ,sizeof(GrRectangle)
        ,sizeof(GrPolygon)
        ,sizeof(GrCircle)
        };

/* Definition der Resourcen-Liste */


static XtResource resources[] = {
{
 	XmNfontList,
 	XmCFontList,
 	XmRFontList,
 	sizeof(XmFontList),
 	offset(GrForm.font_list),
 	XmRString, 
 	"False"
  },
{
 	GrNevent,
 	GrCEvent,
 	XmRBoolean, 
 	sizeof(Boolean),
 	offset(GrForm.grevent),
 	XtRImmediate,
 	NULL
  },
                
};




/****************************************************************
 *
 * Full class record constant
 *
 ****************************************************************/


externaldef( xmgrformclassrec) XmGrFormClassRec xmGrFormClassRec =
{
   {			/* core_class fields      */
      (WidgetClass) &xmFormClassRec,	        /* superclass         */
      "XmGrForm",				/* class_name         */
      sizeof(XmGrFormRec),			/* widget_size        */
      NULL,                                     /* ClassInitialize */	
      NULL,                                     /* ClassPartInitialize */			
      FALSE,					/* class_inited       */
      Initialize,       			/* initialize         */
      NULL,					/* initialize_hook    */
      XtInheritRealize,			        /* realize            */
      NULL,				        /* actions	      */
      0,	                                /* num_actions	      */
      resources,				/* resources          */
      XtNumber(resources),			/* num_resources      */
      NULLQUARK,				/* xrm_class          */
      TRUE,					/* compress_motion    */
      XtExposeCompressMaximal,			/* compress_exposure  */
      FALSE,					/* compress_enterlv   */
      FALSE,					/* visible_interest   */
      Destroy,			                /* destroy            */
      Resize,        			        /* resize             */
      Redisplay,	       		        /* expose             */
      SetValues,                		/* set_values         */
      SetValuesHook,				/* set_values_hook   this is the function to set my own (no resource) values */
      XtInheritSetValuesAlmost,	        	/* set_values_almost  */
      NULL,					/* get_values_hook    */
      NULL,					/* accept_focus       */
      XtVersion,				/* version            */
      NULL,					/* callback_private   */
      XtInheritTranslations,                    /* tm_table           */
      NULL,                    	                /* query_geometry     */
      NULL,             	                /* display_accelerator*/
      NULL                          ,           /* extension          */
   },
   {		/* composite_class fields */
      XtInheritGeometryManager ,    	        /* geometry_manager   */
      ChangeManaged,	                        /* change_managed     */
      XtInheritInsertChild,			/* insert_child       */
      XtInheritDeleteChild,     		/* delete_child       */
      NULL,                                     /* extension          */
   },

   {		/* constraint_class fields */
      NULL,                                     /* constraint resource     */
      0,                                        /* number of constraints   */
       sizeof(XmFormConstraintRec) ,		/* constraint size      */   
      NULL,					/* init proc            */   
      NULL,					/* destroy proc         */   
      NULL,					/* set values proc      */   
      NULL,                                     /* extension            */
   },

   {		/* manager_class fields */
      XtInheritTranslations,                    /* translations           */
      NULL,                                     /* syn_resources          */
      0,                                        /* num_syn_resources      */
      NULL,                                     /* syn_cont_resources     */
      0,                                        /* num_syn_cont_resources */
      XmInheritParentProcess,                   /* parent_process         */
      NULL,                                     /* extension              */
   },

   {                        /* bulletin_board_class fields */
      FALSE,                                    /* always_install_accelerators */
      (XmGeoCreateProc)NULL,                    /* geo_matrix_create  */
      XmInheritFocusMovedProc,                  /* focus_moved_proc   */
      NULL,                                     /* extension          */
   },
   {		/* Form class  */     
    NULL,
   },

   {		/* GrForm class - none */     
      NULL,					/* extension */
   }	
};

externaldef( xmGrFormWidgetClass) WidgetClass xmGrFormWidgetClass
                                  = (WidgetClass) &xmGrFormClassRec ;



/************************************************************************
 *									*
 *  Initialize								*
 *									*
 ************************************************************************/
/* ARGSUSED */
static void 
Initialize( Widget rw, Widget new, ArgList args, Cardinal *num_args ){
    XmGrFormWidget cw = (XmGrFormWidget) new ;

  XGCValues values;
  XtGCMask mask = GCForeground | GCBackground  | GCFont ;

  /* Wenn keine Fontliste durch Resourcen gesetzt ist,laden der Default-Fontliste */
  	if (cw->GrForm.font_list == NULL ){
    		cw->GrForm.font_list =
    		      _XmGetDefaultFontList((Widget) cw, XmLABEL_FONTLIST);
  	}
	if( cw->GrForm.font_list != NULL ){
	    _XmFontListGetDefaultFont(cw->GrForm.font_list,
       			&cw->GrForm.font_struct);
	}
	else
    	    cw->GrForm.font_struct = XLoadQueryFont(XtDisplay(cw),"fixed");
        cw->GrForm.flowtimer = 0;
	  /* Erzeugen des  GCs  */
  	values.foreground = cw->manager.foreground;
  	values.background = cw->core.background_pixel;
  	values.font = cw->GrForm.font_struct->fid;
   	cw->GrForm.gc = XCreateGC(XtDisplay(cw),RootWindowOfScreen(XtScreen((Widget)new))
                , mask, &values);



    	cw->GrForm.flags = 0 ;
    	cw->GrForm.obj = NULL;
        cw->GrForm.Id_counter = 0;
        cw->GrForm.recalc=0;
        cw->GrForm.attached = NULL;
        cw->GrForm.n_attaches = 0;
/*                  add internal event handler to give indication of any vertex hit by cursor */   
        if(cw->GrForm.grevent)     
            XtInsertEventHandler((Widget)cw, PointerMotionMask | ButtonPressMask  
               , False, (XtEventHandler)motion_event , (XtPointer)NULL , XtListHead);         
 }

/************************************************************************
 *									*
 *  Resize    We have to clear the form, use the basic resize function from core
 *            redraw  is done from the expose	                        *
 *									*
 ************************************************************************/
static void Resize( Widget w ){
    XtWidgetProc resize;
   ((XmGrFormWidget)w)->GrForm.recalc++;    /* mark for recalculate graphics */
    if( XtIsRealized(w))
        XClearWindow(XtDisplay(w), XtWindow(w));
    XtProcessLock();
    resize = ((CoreWidgetClass) xmGrFormClassRec.core_class.superclass)->
	core_class.resize;
   XtProcessUnlock();
   (* resize) (w);
    if( XtIsRealized(w))    
        redraw((XmGrFormWidget)w );  /* redraw by hand, at manage of child no expose */
}

/************************************************************************
 *									*
 *  Any of children change managing status 	                        *
 *									*
 ************************************************************************/
 static void ChangeManaged (Widget wid ) {
     XtWidgetProc manage;
    if( XtIsRealized(wid))
        XClearWindow(XtDisplay(wid), XtWindow(wid));
    XtProcessLock();
    manage = ((CompositeWidgetClass) xmGrFormClassRec.core_class.superclass)->
	composite_class.change_managed;   
    XtProcessUnlock();
    (* manage)  (wid);
      
    ((XmGrFormWidget)wid)->GrForm.recalc++;    /* mark for recalculate graphics */ 
    if( XtIsRealized(wid))    
        redraw((XmGrFormWidget)wid );  /* redraw by hand, at manage of child no expose */
}  
        
/************************************************************************
 *									*
 *  Redisplay - 	                                                *
 *									*
 ************************************************************************/
static void Redisplay( Widget wid, XEvent *event, Region region ) {
    
    XmGrFormWidget cw = (XmGrFormWidget) wid ;
    XtExposeProc expose;
    XExposeEvent *xexpose =(XExposeEvent *)event;

 
    XtProcessLock();
    expose = ((CoreWidgetClass) xmGrFormClassRec.core_class.superclass)->
	core_class.expose;   
    XtProcessUnlock();
    (* expose)  (wid, event, region);
    if(!xexpose->count)
        redraw(cw);
}
    
/************************************************************************
 *									*
 *  SetValues	part for  GrNevent, as having a default                 *
 *									*
 ************************************************************************/
static Boolean SetValues ( Widget curr, Widget requ, Widget new, ArgList args, Cardinal *numargs){

  XmGrFormWidget current = (XmGrFormWidget) curr;
  XmGrFormWidget newon   = (XmGrFormWidget) new;
      /* activate internal eventhandler (or remove) */
    if(current->GrForm.grevent && !newon->GrForm.grevent)
         XtRemoveEventHandler(new,  PointerMotionMask | ButtonPressMask 
               , False, (XtEventHandler)motion_event, (XtPointer)NULL);
                                    
    else if(!current->GrForm.grevent && newon->GrForm.grevent)
         XtInsertEventHandler(new, PointerMotionMask | ButtonPressMask 
               , False, (XtEventHandler)motion_event , (XtPointer)NULL , XtListHead);
    return TRUE;
}

/************************************************************************
 *									*
 *  SetValuesHook	handles all special resources, which are setr only
 *                      via SetValues call.                             *
 *									*
 ************************************************************************/
static Boolean SetValuesHook(Widget w, ArgList args,Cardinal *num_args){


    int obj_id, prim_id , vert_no;
    GrObject  *obj = NULL;   


    XmGrFormWidget gw = (XmGrFormWidget)w;
    char types[][16] ={GrNObject,GrNPrimitive,GrNVertex
        ,GrNLineWidth,GrNColor,GrNLineStyle,GrNPositionTyp,GrNVisible
            ,GrNArrow,GrNArrowSize,GrNEffect,GrNFlowIncrement,GrNDashes
        ,GrNx,GrNy,GrNAttachTyp,GrNAttachWidgetX,GrNAttachWidgetY
        ,""};
    int i=0 , k , s1,d1;
    
        for ( k = 0; k < *num_args; k++ ){
            
             for(i=0;types[i][0];i++){
                if(!strcmp(types[i],args[k].name)){
                    switch (i){
                        case 0:     /* Object */
                            obj_id = args[k].value;
                            gw->GrForm.actobj = Get_Object(gw, obj_id);
                            gw->GrForm.actprim=NULL;
                            gw->GrForm.actvert=NULL;
                            if(!obj){
                                continue;
                            }
                            
                            break;
                        case 1:     /* Primitive */
                            prim_id =  args[k].value;
                             gw->GrForm.actprim = Get_Primitive( gw->GrForm.actobj , prim_id);
                                /* vertex = NULL */
                            if(!gw->GrForm.actprim)
                                continue;
                            gw->GrForm.actvert=NULL;
                            break;
                        case 2:    /* Vertex */
                            vert_no =  args[k].value;
                            if(gw->GrForm.actprim && vert_no < gw->GrForm.actprim->npoints)
                                gw->GrForm.actvert = &(gw->GrForm.actprim->points[vert_no]);
                            break;
                        case 3:    /*LineWidth */
                            if(gw->GrForm.actprim)
                                gw->GrForm.actprim->linewidth =  args[k].value;
                            else
                                gw->GrForm.actobj->linewidth =  args[k].value;
                            break;
                        case 4:    /*Color */
                            if(gw->GrForm.actprim)
                                gw->GrForm.actprim->color =  args[k].value;
                            else
                                gw->GrForm.actobj->color =  args[k].value;
                            break;
                         case 5:    /*LineStyle */
                            if(gw->GrForm.actprim)
                                gw->GrForm.actprim->linestyle =  args[k].value;
                            else
                                gw->GrForm.actobj->linestyle =  args[k].value;
                            break;
                        case 6:    /*GrNPositionTyp */
                            if(!gw->GrForm.actvert)
                                continue;
                            if(args[k].value == GR_RELATIVE)
                                gw->GrForm.actvert->flag &= ~GR_ABSOLUTE;
                            else
                                gw->GrForm.actvert->flag |= GR_ABSOLUTE;
                            gw->GrForm.recalc++;
                            break;
                        case 7:    /*Visible */
                            if(gw->GrForm.actprim)
                                gw->GrForm.actprim->visible =  args[k].value;
                            else
                                gw->GrForm.actobj->visible =  args[k].value;
                            break;
                        case 8:    /* GrNArrow */
                            if(gw->GrForm.actprim &&  (gw->GrForm.actprim->Type
                                    == GR_LINE || gw->GrForm.actprim->Type == GR_LINES)){
                                if((args[k].value & (GRBEGIN|GREND)) == Noarr)
                                    gw->GrForm.actprim->flag &= ~(GRBEGIN|GREND);
                                else
                                   gw->GrForm.actprim->flag |=  (args[k].value & (GRBEGIN|GREND));
                            }
                             break;
                       case 9:     /* GrNArrowSize */
                            if(gw->GrForm.actprim &&  (gw->GrForm.actprim->Type
                                    == GR_LINE || gw->GrForm.actprim->Type == GR_LINES))
                            ((GrLines *)(gw->GrForm.actprim))->arrowsize = args[k].value;
                        break;
                       case 10:     /* GrNEffect */
                           if( gw->GrForm.actobj)
                               gw->GrForm.actobj->effect = args[k].value;
                           break;
                        case 11 :   /* Flowincrement */
                            if( gw->GrForm.actobj) {                              
                                gw->GrForm.actobj->flincrem = args[k].value;
                                gw->GrForm.actobj->fllimit = gw->GrForm.actobj->dashes[0]+gw->GrForm.actobj->dashes[1];
                                if( gw->GrForm.actobj->flincrem < 0)
                                    gw->GrForm.actobj->fllimit = -gw->GrForm.actobj->fllimit;
                        }
                            break;
                        case 12:    /* dashes */
                            if( gw->GrForm.actobj) {  
                                for(s1 =0,d1=0;s1<4;s1++){
                                    if((args[k].value >> 24)&0xff)
                                        gw->GrForm.actobj->dashes[d1++]= (args[k].value >> 24)&0xff;
                                    args[k].value <<= 8;
                                }
                                gw->GrForm.actobj->fllimit = gw->GrForm.actobj->dashes[0]+gw->GrForm.actobj->dashes[1];
                                if( gw->GrForm.actobj->flincrem < 0)
                                    gw->GrForm.actobj->fllimit = -gw->GrForm.actobj->fllimit;
                            }
                            break;
                        case 13:     /* XmNx */
                            if(!gw->GrForm.actvert)
                                continue;
                            gw->GrForm.actvert->x = args[k].value;
                            gw->GrForm.recalc++;
                            break;
                        case 14:     /* XmNy */        
                            if(!gw->GrForm.actvert)
                                continue;
                            gw->GrForm.actvert->y = args[k].value;
                            gw->GrForm.recalc++;
                            break;
                        case 15:    /* attachtyp */
                            if(!gw->GrForm.actvert)
                                continue;
                            gw->GrForm.actvert->flag &= ~24;
                            gw->GrForm.actvert->flag &= ~6;
                            gw->GrForm.actvert->flag |= args[k].value &(24|6);
                            gw->GrForm.recalc++;
                            break;
                        case 16:    /* attachwidget X */
                            if(!gw->GrForm.actvert)
                                continue;
                            gw->GrForm.actvert->attached_x = (Widget)args[k].value;
                            gw->GrForm.recalc++;
                            break;
                        case 17:    /* attachwidget Y */
                            if(!gw->GrForm.actvert)
                                continue;
                            gw->GrForm.actvert->attached_y = (Widget)args[k].value;
                            gw->GrForm.recalc++;
                            break;

                   }
                    break;
                }
            }        
        }
    return TRUE;
}        


static void     Destroy(Widget w ){
  XmGrFormWidget gw = ( XmGrFormWidget )w;
   GrObject        *obj = gw->GrForm.obj , *obfree;
   GrPrimitive *prim , *prfree;    
    if(gw->GrForm.flowtimer){
        XtRemoveTimeOut(gw->GrForm.flowtimer);
        gw->GrForm.flowtimer = 0;
    }  
    XFreeGC(XtDisplay( gw), gw->GrForm.gc);
    while(obj){
        
        prim = obj->chain;
        while(prim){
            free(prim->points);
            prfree=prim;
            prim = prim->next;
            free(prfree);
        }
        obfree=obj;
        obj = obj->next;
        free(obfree);
    }
    gw->GrForm.obj = NULL;
    if(gw->GrForm.grevent)
          XtRemoveEventHandler((Widget)gw, PointerMotionMask | ButtonPressMask 
                , False, (XtEventHandler)motion_event , (XtPointer)NULL);     
                   

}
/****************************************************************  */
/*   Static functions for Graphics    */

static void redraw (XmGrFormWidget cw){
   GrObject        *obj = cw->GrForm.obj;
    
    if(cw->GrForm.recalc)
        GrRecalculate(cw );
    while(obj){
        GrDrawObject((Widget)cw , obj , NOEFFECT);   /* set effect to just no, draw all */
        obj = obj->next;
    }
}

static void flow(Widget cw) {
   XmGrFormWidget w = ( XmGrFormWidget )cw;
   GrObject        *obj = w->GrForm.obj;
 
       if(!w->GrForm.flowtimer )
            return;
       if(!obj)
           return;
        while(obj){
            if(obj->effect ) {
                GrDrawObject(cw , obj,GRFLOW);
                obj->floffset += obj->flincrem;
                if ((obj->flincrem > 0 && obj->floffset > obj->fllimit)
                || (obj->flincrem < 0 && obj->floffset < obj->fllimit))
                    obj->floffset = 0;
            }
            obj = obj->next;            
      }
      if(w->GrForm.offset-- < -16) 
           w->GrForm.offset =0;
      w->GrForm.flowtimer = XtAppAddTimeOut(XtWidgetToApplicationContext(cw),
	    100 , (XtTimerCallbackProc)flow , w);
}     

static void  check_attached (XmGrFormWidget w ,GrVertex *v){
  int i , fr = -1;
   
    if((v->flag & (GR_X_RIGHT|GR_X_MIDDLE|GR_X_LEFT)) && v->attached_x
            && !v->attached_x->core.being_destroyed 
               && XtIsManaged(v->attached_x)){
        if(!w->GrForm.n_attaches){
             w->GrForm.attached = (Widget *)calloc(AttachesIncrement,sizeof(Widget));
             w->GrForm.n_attaches =  AttachesIncrement;
        }   
        for(i=0;i<w->GrForm.n_attaches;i++){
            if(w->GrForm.attached[i] == v->attached_x)
                break;
            if(w->GrForm.attached[i] == NULL && fr < 0)
                fr = i;
        }
        if(fr >= 0){
            w->GrForm.attached[fr] = v->attached_x;    /*   need to expand the array!! */
	    XtAddCallback(v->attached_x,XmNdestroyCallback 
                ,(XtCallbackProc)DestroyfOfChild,(XtPointer)w );
        }        
         fr = -1;
    }   
    if((v->flag & (GR_Y_TOP|GR_Y_MIDDLE|GR_Y_BOTTOM)) && v->attached_y
            && !v->attached_y->core.being_destroyed 
               && XtIsManaged(v->attached_y)){
        if(!w->GrForm.n_attaches){
            w->GrForm.attached = (Widget *)calloc(AttachesIncrement,sizeof(Widget));
            w->GrForm.n_attaches =  AttachesIncrement;
        }
        for(i=0;i<w->GrForm.n_attaches;i++){
            if(w->GrForm.attached[i] == v->attached_y)
                break;
            if(w->GrForm.attached[i] == NULL && fr < 0)
                fr = i;
        }
        if(fr >= 0){
            w->GrForm.attached[fr] = v->attached_y;    /*   need to expand the array!! */
	    XtAddCallback(v->attached_y,XmNdestroyCallback 
                ,(XtCallbackProc)DestroyfOfChild,(XtPointer)w );
        }
    }   
        
}

static void ClearAttach (Widget parent,GrVertex *v){
    
    if(v->attached_x == parent)
        v->attached_x = NULL;
    if(v->attached_y == parent)
        v->attached_y = NULL;
}    

static void DestroyfOfChild (Widget parent, XmGrFormWidget w , XmAnyCallbackStruct * cbs ){
   int i ;
  GrObject  *obj = w->GrForm.obj;
  GrPrimitive *prim;
    
        for(i=0;i<w->GrForm.n_attaches;i++){
            if(w->GrForm.attached[i] == parent){
                w->GrForm.attached[i] = NULL;
                break; 
            }
        }
        while(obj){
            prim = obj->chain;
            while(prim){
                for(i=0;i<prim->npoints;i++)
                    ClearAttach (parent , &prim->points[i]);
                prim = prim->next;
            }
            obj = obj->next;
        }
}  

/*
    internal eventhandler to set actual vertex is hit by cursorr
*/  
static void motion_event (Widget w, XtPointer clientData, XEvent *event, Boolean *continueToDispatch){
  XmGrFormWidget gw = ( XmGrFormWidget )w;
  XGrFormEvent  *Gevent = (XGrFormEvent *)event;
  /* int wid = (float)w->core.width / 100.0 , hei = (float)w->core.height / 100.0; */
  
   GrObject        *obj = gw->GrForm.obj ;
   GrPrimitive *prim ;
   GrVertex   *vert;   
   int i;
   
   strcpy(Gevent->Mark,GrEvent);            /* mark active event handler */
  Gevent->objid = Gevent->primid = -1;
  Gevent->obj = NULL;
  Gevent->prim = NULL;
    while(obj){
        prim = obj->chain;
        while(prim){
            if((i=ishit(w,prim,&vert,event->xbutton.x,event->xbutton.y)) >= 0){
           
                     Gevent->objid = obj->Ident;
                     Gevent->primid = prim->Ident;
                     Gevent->obj = obj;
                     Gevent->prim = prim;
                     Gevent->vert = vert;
                     Gevent->vertidx = i;
                     return;
             }
             prim = prim->next;
         }
         obj = obj->next;
     }
}


/*
    check if any vertex was hit from cursor
*/
static int ishit (Widget w, GrPrimitive *prim , GrVertex **vert,int x , int y){
  int wid = 3/* (float)w->core.width / 100.0 */ , hei = 3/* (float)w->core.height / 100.0 */;
   int i;
    *vert = NULL;
    switch(prim->Type){
        case GR_LINE:
        case GR_LINES:
        case GR_POLYGON:
            for (i=0;i<prim->npoints;i++){
                if((prim->points[i].p_x > x-wid 
                        && prim->points[i].p_x < x+wid)
                        && (prim->points[i].p_y > y-hei 
                        && prim->points[i].p_y < y+hei)){
                    *vert = &(prim->points[i]);
                    return i;
                }  
            }
            break;
       case GR_RECTANGLE :
            if((prim->points[0].p_x > x-wid 
                        && prim->points[0].p_x < x+wid)
                        && (prim->points[0].p_y > y-hei 
                        && prim->points[0].p_y < y+hei)){
                *vert=&prim->points[0];
                return 0;
            }
            if((prim->points[1].p_x > x-wid 
                        && prim->points[1].p_x < x+wid)
                        && (prim->points[1].p_y > y-hei 
                        && prim->points[1].p_y < y+hei)){
                *vert=&prim->points[1];
                return 1;
            }
            break;
        case GR_CIRCLE:  
            if((prim->points[0].p_x > x-wid 
                        && prim->points[0].p_x < x+wid)
                        && (prim->points[0].p_y > y-hei 
                        && prim->points[0].p_y < y+hei)){
                *vert=&prim->points[0];
                return 0;
            }
            i= sqrt((double)((x-prim->points[0].p_x)*(x-prim->points[0].p_x)
                   + (y-prim->points[0].p_y)*(y-prim->points[0].p_y)));
            if((i < (prim->points[1].p_x+prim->points[1].p_y+wid+hei)/2 )
                    && (i > (prim->points[1].p_x+prim->points[1].p_y-wid-hei)/2 )){
                    *vert = &prim->points[1];
                    return 1; 
                }                    
            break;
          
    }
    return -1;
}
static void draw_grid (Widget w, GrPrimitive *prim){
    float delta , pos;
    int  wi = w->core.width , h = w->core.height;
    GrGrid *grid = (GrGrid *)prim;
          	
        XSetLineAttributes(XtDisplay(w),((XmGrFormWidget)w)->GrForm.gc
             ,0 , LineDoubleDash, CapRound,JoinRound);
    	delta = (float)wi / (float)grid->nx;
	
	pos = delta;
	while (pos < wi){
	    XDrawLine(XtDisplay(w),XtWindow(w),((XmGrFormWidget)w)->GrForm.gc
		    ,(int)pos,0,(int)pos,h);
	    pos += delta;
	}
	delta = (float)h/ (float)grid->ny;
	pos = delta;
	while (pos < h){
	    XDrawLine(XtDisplay(w),XtWindow(w),((XmGrFormWidget)w)->GrForm.gc
		    ,0,(int)pos,wi,(int)pos);
	    pos += delta;
	}
}
        

static void GrRecalculateVertex(XmGrFormWidget w , GrVertex *v ){
   float wid = (float)w->core.width / 100.0 , hei = (float)w->core.height / 100.0;
    /*   check flags for attachments and do it now depending of that */
   
   check_attached (w ,v);
   
   if((v->flag & GR_ABSOLUTE) == GR_RELATIVE){
       if (Calc_X(v) )
            v->p_x = wid * (float)v->x;
       if ( Calc_Y(v) )
            v->p_y = hei * (float)v->y;
   }else if((v->flag & 1) == GR_ABSOLUTE){
        if (Calc_X(v))
           v->p_x = v->x;
       if (Calc_Y(v))
            v->p_y = v->y;
   }
      /* attached coord, use users coords as offsets */
   if ( v->flag & (GR_X_RIGHT|GR_X_MIDDLE|GR_X_LEFT) ){        /* X-Attached */
       if(v->attached_x && !v->attached_x->core.being_destroyed 
               && XtIsManaged(v->attached_x)){
           switch (v->flag & 0x06){
                case  GR_X_LEFT :
                    v->p_x =  v->attached_x->core.x 
                            + v->p_x;
                    break;
                case  GR_X_MIDDLE:
                    v->p_x =  v->attached_x->core.x + v->attached_x->core.width/2
                             + v->p_x;
                    break;
                case GR_X_RIGHT:
                    v->p_x =  v->attached_x->core.x + v->attached_x->core.width
                             + v->p_x;
                    break;
           }
       }else
            w->GrForm.recalc++;
   }    
   if ( v->flag & (GR_Y_TOP|GR_Y_MIDDLE|GR_Y_BOTTOM) ){        /* Y-Attached  */
       if(v->attached_y &&  !v->attached_y->core.being_destroyed 
               && XtIsManaged(v->attached_y)){
           switch (v->flag & 0x18){
                case  GR_Y_TOP :
                    v->p_y =  v->attached_y->core.y
                             + v->p_y;
                    break;
                case  GR_Y_MIDDLE:
                    v->p_y =  v->attached_y->core.y + v->attached_y->core.height/2
                             + v->p_y;
                    break;
                case GR_Y_BOTTOM:
                    v->p_y =  v->attached_y->core.y + v->attached_y->core.height
                             + v->p_y;
                    break;
           }
       }else
            w->GrForm.recalc++;
   }                                 
}

static void GrRecalculate(XmGrFormWidget w ){
   GrObject        *obj = w->GrForm.obj;
   GrPrimitive *prim;    
   int i;
     
    while(obj){
        
        prim = obj->chain;
        while(prim){
            for (i= 0 ; i< prim->npoints;i++)
                GrRecalculateVertex(w,&prim->points[i]);
                
            prim = prim->next;
        }
        obj = obj->next;
    }
    w->GrForm.recalc = 0;   
}

    /* additional arrow at begin or end of line */
static void arrow (Widget w ,GrVertex *beg , GrVertex *end , int length , int flag  ){
 XPoint poi[4];
int i;
float sin_a,cos_a , x,y;
float l,xl,yl;


    if(flag & 1){               /* arrow at begin */

        xl = beg->p_x - end->p_x;
        yl = beg->p_y - end->p_y;
        poi[0].x = beg->p_x;
        poi[0].y = beg->p_y;
    }else{                      /* arrow at end */

        xl = end->p_x - beg->p_x;
        yl = end->p_y - beg->p_y;
        poi[0].x = end->p_x;
        poi[0].y = end->p_y;
    }
    l = sqrt((double)(xl*xl)+(double)(yl*yl));
    sin_a = yl/l;
    cos_a = xl/l;


    poi[1].x = -length;
    poi[1].y =  (length/2) ?-length/2 : -1;
    poi[2].x =  length/3;
    poi[2].y =  (length/2) ? length/2 : 1;
    poi[3].x = -length/3;
    poi[3].y =   (length/2) ? length/2 : 1;

    for(i=1;i<4;i++){           /* rotate it to centering with given line */
        x = poi[i].x * cos_a - poi[i].y * sin_a;
        y = poi[i].y * cos_a + poi[i].x * sin_a;
        poi[i].x = x;
        poi[i].y = y;
    }



    XFillPolygon(XtDisplay(w),XtWindow(w),((XmGrFormWidget)w)->GrForm.gc,poi,4
        ,Complex, CoordModePrevious);

}
    

static void GrDrawObject(Widget w , GrObject *obj , int effect){
    int i;
    GrPrimitive *prim = obj->chain;
    unsigned int lstyle,lwidth=0  ;
    char fdashes[2] = {8,8};
    Pixel forground;
    if(!obj->visible)
        return;
    ++(((XmGrFormWidget)w)->GrForm.cntflag);
    while(prim){
        lstyle = obj->linestyle;
        lwidth = obj->linewidth;        
        if(effect && (obj->effect == GRBLINK || prim->effect == GRBLINK) 
                && ((((XmGrFormWidget)w)->GrForm.cntflag) &2 )){
            XSetForeground(XtDisplay(w),((XmGrFormWidget)w)->GrForm.gc
                ,w->core.background_pixel);
             forground = w->core.background_pixel;
       }else if(prim->color){
            forground = prim->color;
            XSetForeground(XtDisplay(w),((XmGrFormWidget)w)->GrForm.gc
                ,prim->color);
        }else{
            if(obj->color)
                forground = obj->color;
            else
                forground = ((XmGrFormWidget)w)->manager.foreground;
            XSetForeground(XtDisplay(w),((XmGrFormWidget)w)->GrForm.gc,obj->color);
        }
        if(prim->linewidth != -1)
            lwidth = prim->linewidth;
        if(prim->linestyle != -1)
            lstyle = prim->linestyle;
        if(effect && (obj->effect == GRFLOW || prim->effect == GRFLOW))
            lstyle=2;    /* for flow enforce ddash  8 8 linestyle */
        XSetLineAttributes(XtDisplay(w),((XmGrFormWidget)w)->GrForm.gc
             ,lwidth , lstyle, CapRound,JoinRound);
        if(effect && (obj->effect == GRFLOW || prim->effect == GRFLOW))
            XSetDashes(XtDisplay(w), ((XmGrFormWidget)w)->GrForm.gc
            , ((XmGrFormWidget)w)->GrForm.offset, fdashes,2);
        else if(obj->dashes[0])
            XSetDashes(XtDisplay(w), ((XmGrFormWidget)w)->GrForm.gc
            , 0, obj->dashes,2);
            
        if(prim->visible){
            switch (prim->Type){
                default:
                    break;
                case GR_GRID:
                    draw_grid ( w, prim);
                    break;
                case GR_LINE:
                    if((prim->flag & GRBEGIN) == GRBEGIN)
                        arrow (w ,&prim->points[0] , &prim->points[1]  , ((GrLine *)prim)->arrowsize , 1  );
                     if((prim->flag & GREND) == GREND)
                        arrow (w ,&prim->points[0] , &prim->points[1]  , ((GrLine *)prim)->arrowsize , 0  );
                              
                    XDrawLine(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                        , prim->points[0].p_x, prim->points[0].p_y, prim->points[1].p_x, prim->points[1].p_y);
                    break;    
                           
                case GR_LINES: case GR_POLYGON :
{  
  XPoint *poi = (XPoint *)calloc(prim->npoints,sizeof(XPoint));  /* alloc a converted copy of points */
                    for(i=0;i<prim->npoints;i++){
                        poi[i].x = prim->points[i].p_x;
                        poi[i].y = prim->points[i].p_y;
                    }
                    if(prim->Type == GR_POLYGON){
                        XFillPolygon(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                             , poi, prim->npoints,Complex , CoordModeOrigin );
                    }else{    
                    
                        if((prim->flag & GRBEGIN) == GRBEGIN)
                            arrow (w ,&prim->points[0] , &prim->points[1]  
                                , ((GrLines *)prim)->arrowsize , 1  );
                        if((prim->flag & GREND) == GREND)
                            arrow (w ,&prim->points[prim->npoints -2] , &prim->points[prim->npoints -1]  
                                 , ((GrLines *)prim)->arrowsize , 0  );
                        XDrawLines(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                             , poi, prim->npoints,CoordModeOrigin );
                    }                                                                 
                    free(poi);
}                 
                     break;
                                 
                case GR_RECTANGLE:
{     /* first do correction for bad defined rectangles */
    int x=prim->points[0].p_x ,y= prim->points[0].p_y, wid = prim->points[1].p_x - prim->points[0].p_x,hei=prim->points[1].p_y - prim->points[0].p_y;    
                        if(wid < 0){
                             x += wid;
                             wid = -wid;
                        }
                        if(hei < 0){
                             y += hei;
                             hei = -hei;
                        }
                                              
                        if(prim->flag)
                            XFillRectangle(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                                , x, y, wid, hei);
                        else
                            XDrawRectangle(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                                , x, y , wid, hei); 
}                     
                    break;  
               case GR_CIRCLE :
        /* DrawArc positions and sizes the surrounding rectangle, not the center and Radius!!!! */
                     if(prim->flag & 1)
                        XFillArc(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                                 ,prim->points[0].p_x - prim->points[1].p_x, prim->points[0].p_y - prim->points[1].p_y
                                 , prim->points[1].p_x*2, prim->points[1].p_y*2
                                 ,((GrCircle *)prim)->startangle ,((GrCircle *)prim)->rangeangle );
                     else {
                        XDrawArc(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                                 ,prim->points[0].p_x - prim->points[1].p_x, prim->points[0].p_y - prim->points[1].p_y
                                 , prim->points[1].p_x*2, prim->points[1].p_y*2
                                 ,((GrCircle *)prim)->startangle ,((GrCircle *)prim)->rangeangle );
                        if(prim->flag > 1){
int seg = 360/(prim->flag & ~1) , star =0;
                            XSetForeground(XtDisplay(w),((XmGrFormWidget)w)->GrForm.gc
                                    ,((XmGrFormWidget)w)->core.background_pixel);
                            XFillArc(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                                 ,prim->points[0].p_x - prim->points[1].p_x
                                 , prim->points[0].p_y - prim->points[1].p_y
                                 , prim->points[1].p_x*2, prim->points[1].p_y*2
                                 ,((GrCircle *)prim)->startangle ,((GrCircle *)prim)->rangeangle );
                            
                            XSetForeground(XtDisplay(w),((XmGrFormWidget)w)->GrForm.gc
                                    ,forground);
                            if(obj->effect == GRFLOW ){
                                star = (((GrCircle *)prim)->floffset * (360/(seg) ));
                                ((GrCircle *)prim)->floffset += obj->flincrem;
                                if(star > 2*seg 
                                        || (star < -2*seg))
                                        ((GrCircle *)prim)->floffset = star = 0;  
                            }         
                            for(i=0;i< (prim->flag & ~1) ; i +=2)
                                    XFillArc(XtDisplay(w), XtWindow(w), ((XmGrFormWidget)w)->GrForm.gc
                                        ,prim->points[0].p_x - prim->points[1].p_x
                                        , prim->points[0].p_y - prim->points[1].p_y
                                        , prim->points[1].p_x*2, prim->points[1].p_y*2
                                        ,(i*seg + star) *64 ,seg*64 );
                        } 
                    }                              
                    break;                 
           } 
                 
        }
        if(prim->linewidth)   /* reset linewidth */
            XSetLineAttributes(XtDisplay(w),((XmGrFormWidget)w)->GrForm.gc
             ,obj->linewidth ,  LineSolid,CapRound,JoinRound);

        prim=prim->next;
    }   
}
static void Add_Primitive(GrObject *obj , GrPrimitive *p){
    GrPrimitive *prim = obj->chain , **new = &obj->chain;
    
   while(prim){
       new = &prim->next;
       prim = prim->next;
   }
   *new = p;
}
    

static GrObject *Get_Object(XmGrFormWidget w , int Id){
   GrObject        *obj = w->GrForm.obj;
   
    while(obj){
        if(obj->Ident  == Id)
            return obj;
        obj = obj->next;
    }
    return NULL;
}

static GrPrimitive *Get_Primitive( GrObject *obj , int prim_id){
   GrPrimitive  *prim = obj->chain;
   
    while(prim){
        if(prim->Ident  == prim_id)
            return prim;
        prim = prim->next;
    }
    return NULL;
}

static int GrPoly(Widget w,int object,GrVertex *points, int npoints , int flag ) {
/* flag = 0 -> just lines
   flag = 1 -> polygon
   flag = 2 -> filled polygon
*/ 
  GrLines *drw;
  GrPrimitive *pdrw ;
  GrObject *obj;

    if( ! XmIsGrForm(w) ){
       return -1;
   }
   if(!(obj = Get_Object((XmGrFormWidget)w , object)))
       return -1;
   drw = (GrLines *)calloc(1,sizeof(GrLines));
   pdrw = (GrPrimitive *)drw ;
   pdrw->visible = 1;      /* create as visible (object overrides it) */
   pdrw->linestyle = -1;
   pdrw->linewidth = -1;
   pdrw->Ident = ++(((XmGrFormWidget)w)->GrForm.Id_counter);
   pdrw->points = (GrVertex *)calloc(npoints,sizeof(GrVertex));
   pdrw->npoints = npoints;
   memcpy(pdrw->points , points, sizeof(GrVertex) * npoints);
   switch (flag){
       case 0:
        pdrw->Type = GR_LINES;
        drw->arrowsize = ArrowSize;
        break;
       case 1:       /* filled polygon */
        pdrw->Type = GR_POLYGON;
        ((GrPolygon *)pdrw)->fill =1;
        break;
   }  
   Add_Primitive(obj , pdrw);
   ((XmGrFormWidget)w)->GrForm.recalc++;
   ReDraw(w)
   return pdrw->Ident;
}

 
static int GrRect(Widget w,int object
        , int x, int y, int width, int height , int flag, int Mode){

  GrRectangle *drw ;
  GrPrimitive *pdrw;
  GrObject *obj;
     
    if( ! XmIsGrForm(w) ){
       return -1;
   }
   if(!(obj = Get_Object((XmGrFormWidget)w , object)))
       return -1;
   drw = (GrRectangle *)calloc(1,sizeof(GrRectangle));
   pdrw = (GrPrimitive *)drw ;
   pdrw->points = (GrVertex *)calloc(2,sizeof(GrVertex));
   pdrw->npoints = 2;
   pdrw->points[0].x = x;
   pdrw->points[0].y = y;
   pdrw->points[1].x = x+width;
   pdrw->points[1].y = y+height;
   pdrw->linestyle = -1;
   pdrw->linewidth = -1;
   pdrw->points[1].flag = pdrw->points[0].flag = Mode;
   pdrw->flag = flag;
   pdrw->Type = GR_RECTANGLE;
   pdrw->visible = 1;      /* create as visible (object overrides it) */
   pdrw->Ident = ++(((XmGrFormWidget)w)->GrForm.Id_counter);
   Add_Primitive(obj , pdrw);
   ((XmGrFormWidget)w)->GrForm.recalc++;
   ReDraw(w)
   return pdrw->Ident;
   
}
/*   Public functions for Graphics    */

int GrFormDrawLine(Widget w,int object
        , int x1, int y1, int x2, int y2 , int Mode){
  GrLine *drw ;
  GrPrimitive *pdrw;
  GrObject *obj;
     
    if( ! XmIsGrForm(w) ){
       return -1;
   }
   if(!(obj = Get_Object((XmGrFormWidget)w , object)))
       return -1;
   
   drw = (GrLine  *)calloc(1,sizeof(GrLine ));
   pdrw = (GrPrimitive *)drw ;
   pdrw->points = (GrVertex *)calloc(2,sizeof(GrVertex));
   pdrw->npoints = 2;
   pdrw->points[0].x = x1;
   pdrw->points[0].y = y1;
   pdrw->points[1].x = x2;
   pdrw->points[1].y = y2;
   pdrw->linewidth = -1;
   pdrw->linestyle = -1;
   pdrw->points[1].flag = pdrw->points[0].flag = Mode;
   pdrw->Type = GR_LINE;
   drw->arrowsize = ArrowSize;
   pdrw->visible = 1;      /* create as visible (object overrides it) */
   pdrw->Ident = ++(((XmGrFormWidget)w)->GrForm.Id_counter);
   Add_Primitive(obj , pdrw);
   ((XmGrFormWidget)w)->GrForm.recalc++;
   ReDraw(w)
   return pdrw->Ident;
   
}


 
int GrFormDrawLines(Widget w,int object,GrVertex *points, int npoints) {
     
     return GrPoly( w, object,points, npoints , 0 );

}
int GrFormFillPolygon(Widget w,int object,GrVertex *points, int npoints) {
     
     return GrPoly( w, object,points, npoints , 1 );

}

/*    fill =0  circle only, fill = 1 filled  cicle
      fill > 1 n-segments filled with circle */
 int GrFormCircle (Widget w,int object,int x_center,int y_center,int radius 
         , int fill, int Mode) {
  GrCircle *drw;
  GrPrimitive *pdrw  ;
  GrObject *obj;

    if( ! XmIsGrForm(w) ){
       return -1;
   }
   if(!(obj = Get_Object((XmGrFormWidget)w , object)))
       return -1;
   drw = (GrCircle *)calloc(1,sizeof(GrCircle));
   pdrw = (GrPrimitive *)drw ;
   pdrw->Type = GR_CIRCLE;
   pdrw->visible = 1;      /* create as visible (object overrides it) */
  
   pdrw->Ident = ++(((XmGrFormWidget)w)->GrForm.Id_counter);
   pdrw->points = (GrVertex *)calloc(2,sizeof(GrVertex));
   pdrw->npoints = 2;
   pdrw->points[0].x =  x_center;
   pdrw->points[0].y = y_center;
   pdrw->points[1].x = pdrw->points[1].y = radius;
   pdrw->points[0].flag = Mode;
   pdrw->points[1].flag = GR_ABSOLUTE;        /* radius is always absolute */
   pdrw->linestyle = -1;
   pdrw->linewidth = -1;
   drw->startangle = 0;
   drw->rangeangle = 360*64;    /* full circle */
   pdrw->points[1].flag &= 1;   /* cannot attach the radius */
   pdrw->points[1].attached_x = pdrw->points[1].attached_y = NULL;
   pdrw->flag = fill;
   Add_Primitive(obj , pdrw);
   ((XmGrFormWidget)w)->GrForm.recalc++;
   ReDraw(w)
   return pdrw->Ident;
}
   
          
int GrFormRectangle(Widget w,int object
        , int x, int y, int width, int height , int Mode){
    return GrRect(w , object, x, y, width, height, 0,Mode);
}
  
int GrFormFillRectangle(Widget w,int object
        , int x, int y, int width, int height, int Mode){
    return GrRect(w , object, x, y, width, height, 1,Mode);
}

int GrFormGrid (Widget w,int object , int nx , int ny){
  GrGrid *drw;
  GrPrimitive *pdrw  ;
  GrObject *obj; 

      if( ! XmIsGrForm(w) ){
       return -1;
   }
   if(!(obj = Get_Object((XmGrFormWidget)w , object)))
       return -1;
   drw = (GrGrid *)calloc(1,sizeof(GrGrid));
   pdrw = (GrPrimitive *)drw ;
   pdrw->Type = GR_GRID;
   pdrw->visible = 1;      /* create as visible (object overrides it) */
   pdrw->Ident = ++(((XmGrFormWidget)w)->GrForm.Id_counter);
   drw->nx = nx;
   drw->ny = ny;
   pdrw->color = ~w->core.background_pixel;
   Add_Primitive(obj , pdrw);
   ((XmGrFormWidget)w)->GrForm.recalc++;
   ReDraw(w)
   return pdrw->Ident;
}    

/*
        Delete single Vertex 
        in case of only two Vertices, delete Primitive
*/        
int GrFormDeleteV (Widget w, int objid, int primid , int vertidx){
   GrObject  *obj;
   GrPrimitive *prim;
   int i;
    if( ! XmIsGrForm(w) )
       return 0;
    if(!(obj = Get_Object((XmGrFormWidget)w , objid)))
       return 0;
    if(!(prim = Get_Primitive( obj , primid)))
        return 0;
    if(prim->npoints <= 2)
        return GrFormDeleteP (w , objid , primid);
    if(((XmGrFormWidget)w)->GrForm.actvert == &prim->points[vertidx])
        ((XmGrFormWidget)w)->GrForm.actvert = NULL;
    for(i=vertidx ; i<prim->npoints-1;i++)
        prim->points[i] = prim->points[i+1];
    prim->npoints--;
    ReDraw(w)
    return 1;    
}

/*
    Delete a Primitive
*/
    
int GrFormDeleteP (Widget w, int objid, int primid ){
   GrObject  *obj;
   GrPrimitive *prim, *nprim;
    if( ! XmIsGrForm(w) )
       return 0;
    if(!(obj = Get_Object((XmGrFormWidget)w , objid)))
       return 0;
    if(!(prim = Get_Primitive( obj , primid)))
        return 0;
   
    free(prim->points);
    nprim = obj->chain;
    while(nprim && nprim->next != prim)
        nprim = nprim->next;
    if(!nprim)
        obj->chain = prim->next;
    else
        nprim->next = prim->next;
    free (prim);
    if(((XmGrFormWidget)w)->GrForm.actprim == prim){
        ((XmGrFormWidget)w)->GrForm.actprim = NULL;
        ((XmGrFormWidget)w)->GrForm.actvert = NULL;
    }
    ReDraw(w)
    return 2;
}
/*
    Delete an object
*/
int GrFormDeleteO (Widget w, int objid){    
   GrObject  *obj , *nobj;
   GrPrimitive *prim;
    if(!(obj = Get_Object((XmGrFormWidget)w , objid)))
       return 0;
    prim = obj->chain;
    while(prim){
        GrFormDeleteP (w, objid ,prim->Ident);
        prim = obj->chain;
    }
    nobj = ((XmGrFormWidget)w)->GrForm.obj;
    while(nobj && nobj->next != obj)
        nobj = nobj->next;
    if(!nobj)
       ((XmGrFormWidget)w)->GrForm.obj = obj->next;
    else
       nobj->next =  obj->next;
    free(obj);
    if(((XmGrFormWidget)w)->GrForm.actobj == obj ){
        ((XmGrFormWidget)w)->GrForm.actprim = NULL;
        ((XmGrFormWidget)w)->GrForm.actvert = NULL;
        ((XmGrFormWidget)w)->GrForm.actobj = NULL;
    }
    ReDraw(w)
    return 2;
}
    
    
void GrRedraw(Widget w) {
    ReDraw(w) 
}
    
    
int GrForm_New_Object(Widget w) {  
   XmGrFormWidget cw = (XmGrFormWidget) w ; 
   GrObject        *obj = cw->GrForm.obj , **new = &cw->GrForm.obj;
   char dashes[4]={1,1,8,3}; 
    
   if( ! XmIsGrForm(w) ){
       return -1;
   }
   while(obj){
       new = &obj->next;
       obj = obj->next;
   }
   *new = (GrObject *)calloc(1,sizeof(GrObject));
   (*new)->visible = 1;     /* create as visible */
   (*new)->linestyle = LineSolid;
   (*new)->color = cw->manager.foreground;
   (*new)->Ident = ++(cw->GrForm.Id_counter);
   (*new)->effect = NOEFFECT;
   memcpy((*new)->dashes,dashes,4);
   (*new)->flincrem = 1;
   (*new)->fllimit = 11;
   return (*new)->Ident;
}

GrObject *Get_GrForm_Object(Widget w , int id){
  GrObject  *obj , *nobj;
  
    if( ! XmIsGrForm(w) )
       return NULL;
    if(!(obj = Get_Object((XmGrFormWidget)w , id)))
       return NULL;
    
    
    nobj = ( GrObject  *)calloc(1,sizeof(GrObject));
    memcpy(nobj,obj,sizeof(GrObject));
    nobj->next = NULL;
    nobj->chain = NULL;
    return nobj ;
}

void Set_GrForm_Object(Widget w,GrObject *cobj){
   GrObject  *obj;
   
    if( ! XmIsGrForm(w) )
       return ;
    if(!(obj = Get_Object((XmGrFormWidget)w , cobj->Ident)))
       return ;
    obj->color = cobj->color;
    obj->visible = cobj->visible ;
    obj->effect = cobj->effect;
    obj->linestyle = cobj->linestyle;
    obj->Switch_condition = cobj->Switch_condition;

}    
/*
   copied memory to primitive is returned
   The internals depending on the type are original, it is up to user to do only
   LEGAL changes (never touch attaches in vertices !!!!!
*/

GrPrimitive *Get_GrForm_Primitive(Widget w,int obj_id,int prim_id) {
  GrObject  *obj ;
  GrPrimitive *prim, *nprim;
  
    if( ! XmIsGrForm(w) )
       return NULL;
    if(!(obj = Get_Object((XmGrFormWidget)w , obj_id)))
       return NULL;
    
    if(!(prim = Get_Primitive( obj , prim_id)))
        return NULL;
    nprim = (GrPrimitive *)calloc(1,primsizes[prim->Type]);
    
    memcpy(nprim ,prim,primsizes[prim->Type]);
    nprim->next = NULL;
   
    return nprim;
}

void Set_GrForm_Primitive(Widget w, int obj_id, GrPrimitive *cprim){
   GrObject  *obj;
   GrPrimitive *prim , *next;
   int typ;
    if( ! XmIsGrForm(w) )
       return ;
    if(!(obj = Get_Object((XmGrFormWidget)w , obj_id)))
       return ;
    if(!(prim = Get_Primitive( obj , cprim->Ident)))
        return ;
    next = prim->next;
    typ = prim->Type;
    
    memcpy(prim,cprim,primsizes[prim->Type]);
    prim->next = next;
    prim->Type = typ;
    /* this allows the user to change the Vertices  BE CAREFUL */
    ((XmGrFormWidget)w)->GrForm.recalc++;
    
    /* here we need a redraw in case position/attaches have changed */
    ReDraw(w)
}
void GrForm_Effect(Widget w , int start){
    
      if( ! XmIsGrForm(w) )
          return  ;
    if( ((XmGrFormWidget)w)->GrForm.flowtimer){
         XtRemoveTimeOut(((XmGrFormWidget)w)->GrForm.flowtimer);
         ((XmGrFormWidget)w)->GrForm.flowtimer = 0;
     }
    if(start)
        ((XmGrFormWidget)w)->GrForm.flowtimer = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
	    300 , (XtTimerCallbackProc)flow , w);
    ReDraw(w)        
}    
Widget GrCreateForm(Widget parent ,  String name, ArgList argslist, Cardinal argcount ){
      return XtCreateWidget(name,xmGrFormWidgetClass,parent,argslist, argcount);

}  

     
   
#ifdef __cplusplus
}
#endif



