/*
        mindlib/text_funcs.c

        Copyright (C) 1994-97 flux GbR (flux@berlin.snafu.de)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2, or (at your option)
        any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


  /* Alle Routinen beziehen sich NUR auf XmTextField
***********************************************************************************
    Die folgenden Routinen testen einen bestimmen Datentyp.
    Alle Routinen geben TRUE zurueck, wenn der Test erfolgreich war.
    Alle nicht implementierten Routinen liefern immer TRUE. mit ** markiert.
    Alle Routinen, die auf ...Cb enden koennen direkt als 
    ModifyVerifyCallback verwendet werden

    int ModifyVerifyCheckOnFloat(Widget w, XmTextVerifyCallbackStruct *cbs);
    void ModifyVerifyCheckOnFloatCb(Widget w, XtPointer data, XtPointer cbs);
    Testet eine Floatzahl ohne Exponenten

    int ModifyVerifyCheckOnInt(Widget w, XmTextVerifyCallbackStruct *cbs);
    void ModifyVerifyCheckOnIntCb(Widget w, XtPointer data, XtPointer cbs);
    Testet eine beliebig lange Dezimalzahl

    int ModifyVerifyCheckOnHex(Widget w, XmTextVerifyCallbackStruct *cbs);
    void ModifyVerifyCheckOnHexCb(Widget w, XtPointer data, XtPointer cbs);
    Testet eine beliebig lange Hexzahl

    int ModifyVerifyCheckOnOct(Widget w, XmTextVerifyCallbackStruct *cbs);
    void ModifyVerifyCheckOnOctCb(Widget w, XtPointer data, XtPointer cbs);
    Testet eine beliebig lange Oktalzahl

    int ModifyVerifyCheckOnBin(Widget w, XmTextVerifyCallbackStruct *cbs);
    void ModifyVerifyCheckOnBinCb(Widget w, XtPointer data, XtPointer cbs);
    Testet eine beliebig lange Binaerzahl

    int ModifyVerifyCheckOnFloat(Widget w, XmTextVerifyCallbackStruct *cbs);
    void ModifyVerifyCheckOnFloatCb(Widget w, XtPointer data, XtPointer cbs);
    Testet eine Floatzahl ohne Exponenten

    int ModifyVerifyCheckOnLetter(Widget w, XmTextVerifyCallbackStruct *cbs);
    void ModifyVerifyCheckOnLetterCb(Widget w, XtPointer data, XtPointer cbs);
    Testet eine beliegig lange Zeichenkette aus a..z und A..Z

    int ModifyVerifyCheckOnCName(Widget w, XmTextVerifyCallbackStruct *cbs);
    void ModifyVerifyCheckOnCNameCb(Widget w, XtPointer data, XtPointer cbs);
    Testet, ob der EIntrag eine gueltiger C-Name ist. Ohne $, . oder ->

    int ModifyVerifyCheckOnLen(Widget w, XmTextVerifyCallbackStruct *cbs, int MaxLen);
    Testet, ob die zulaessige Textlaenge nicht ueberschtitten wird.

    int ModifyVerifyCheck(Widget w, XmTextVerifyCallbackStruct *cbs, char *Chars);
    Testet, ob im TextFeld nur Zeichen aus Chars vorkommen.

************************************************************************************
************************************************************************************
************************************************************************************
  Weitere Routinen

  void ActivationHex2IntCb(Widget w, XtPointer unused1, XtPointer unused2);
  Wandelt einen HexString in einen Integer um. 
  Zu Verwenden als Acticate, oder LosingFocus-Callback

************************************************************************************
************************************************************************************
************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <XMind.h>

/* STATIC GLOBALS */

static char *password = NULL; 	/* Buffer for storing password */
static int   text_index = 0;         /* Position of next character in buffer */

/* DEFINES */

#ifndef strdup
char *strdup(const char *s);
#endif

/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyResult
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs
 *
 *  Return:	       char *
 *
 *  Documentation: Makes the potential result of a XmTextVerifyCallback
 *
 *
 *  History:       20.2.96 created by flux
 *
 *
 ***************************************************************************/
 
char *ModifyVerifyResult(Widget w, XmTextVerifyCallbackStruct *cbs) {

  char *New,*old,*rem;
  int i,j,len;

  old = XmTextFieldGetString(w);
  rem = old;
  if(!old) { /* No old Text */
    if(cbs->text->ptr) { /* Return the new text as whole text */
      New = (char *)XtMalloc(cbs->text->length+1);
/*
 *       for(i=0;i<cbs->text->length;i++)
 *            New[i] = old[i];
 *    New[cbs->text->length] = 0;
 */
      strcpy(New , cbs->text->ptr);
    } else { /* Nothing to do ??? */
      New = NULL;
    }
  } else {
    len = strlen(old) + cbs->text->length + 1; /* maximal possible length */
    New = (char *)XtMalloc(len);
    j = 0;
    if(cbs->startPos) for(i=0;i<cbs->startPos;i++) New[j++] = old[i];
    if(cbs->text->length) for(i=0;i<cbs->text->length;i++) New[j++] = cbs->text->ptr[i];
    old += cbs->endPos;
    while(*old) New[j++] = *old++;
    New[j] = 0;
  }
  if(rem) XtFree(rem);
  return New;
}
    


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnFloat
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs
 *
 *  Return:	       int
 *
 *  Documentation: Checks if text is a (real) number of [+-][0-9]*[.][0-9]*
 *                 '+9.08'    is valid
 *                 '-.3'      is valid
 *                 '4711'     is valid
 *                 ''         is valid
 *                 ' +9.08'   is invalid (no blanks allowed)
 *                 '0.123E47' is invalid
 *                 '-.'       is valid
 *
 *  History:       20.2.96 created by flux
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheckOnFloat(Widget w, XmTextVerifyCallbackStruct *cbs) {

  char *num, *rem=NULL;
  int ret = TRUE;

/*  num = XmTextFieldGetString(w); */
  num = ModifyVerifyResult(w, cbs);
  if(!num) goto Ready;
  rem = num;
  if(!*num) goto Ready;
  if(*num == '-' || *num == '+') num++;
  if(!*num) goto Ready;
  if(*num == '.') goto NoMantiss;
  while(*num >= '0' && *num <= '9') num++;
  if(!*num) goto Ready;
  if(!(*num == '.')) {
    ret = FALSE;
    goto Ready;
  }
NoMantiss:
  num++;
  while(*num >= '0' && *num <= '9') num++;
  if(*num) ret = FALSE;
Ready:
  XtFree(rem);
  return ret;
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnFloatCb
 *  
 *  Input:         Widget w, XtPointer data, XtPointer cbs
 *
 *  Return:	       void
 *
 *  Documentation: Callback-Funktion, die nur Real-Werte zulaesst
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
void ModifyVerifyCheckOnFloatCb(Widget w, XtPointer data, XtPointer cbs) {
  if(!ModifyVerifyCheckOnFloat(w, (XmTextVerifyCallbackStruct *)cbs)) {
    ((XmTextVerifyCallbackStruct *)cbs)->doit = FALSE;
  }
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheck
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs, char *Chars
 *
 *  Return:	       int
 *
 *  Documentation: Prueft, ob alle Zeichen des TextFeldes in 'Chars' enthalten sind
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheck(Widget w, XmTextVerifyCallbackStruct *cbs, char *Chars) {

  char *New, *rem;

  if(!w || !cbs) return FALSE;
  if(!Chars) return TRUE; /* Alles erlaubt */

  New = ModifyVerifyResult(w, cbs);
  if(!New) return TRUE;

  rem = New;
  while(*New) {
    if(!strchr(Chars, *New++)) {
      XtFree(rem);
      return FALSE;
    }
  }
  XtFree(rem);
  return TRUE;
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnInt
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs
 *
 *  Return:	       int
 *
 *  Documentation: Prueft, ob der Inhalt des TextFeldes ein Integer ist
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheckOnInt(Widget w, XmTextVerifyCallbackStruct *cbs) {

  char *num, *rem;

  if(!w || !cbs) return FALSE;

  num = ModifyVerifyResult(w, cbs);

  if(!num) return TRUE;
  rem = num;

  if(*num == '-' || *num == '+') num++;
  while(*num) {
    if(!strchr("0123456789", *num++)) {
      XtFree(rem);
      return FALSE;
    }
  }
  XtFree(rem);
  return TRUE;
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnIntCb
 *  
 *  Input:         Widget w, XtPointer data, XtPointer cbs
 *
 *  Return:	       void
 *
 *  Documentation: Callback-Funktion, die nur Integer-Werte zulaesst
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
void ModifyVerifyCheckOnIntCb(Widget w, XtPointer data, XtPointer cbs) {
  if(!ModifyVerifyCheckOnInt(w, (XmTextVerifyCallbackStruct *)cbs)) {
    ((XmTextVerifyCallbackStruct *)cbs)->doit = FALSE;
  }
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnHex
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs
 *
 *  Return:	       int
 *
 *  Documentation: Prueft, ob der Inhalt des TextFeldes eine Hex-Zahl ist
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheckOnHex(Widget w, XmTextVerifyCallbackStruct *cbs) {
  return ModifyVerifyCheck(w, cbs, "0123456789ABCDEFabcdef");
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnHexCb
 *  
 *  Input:         Widget w, XtPointer data, XtPointer cbs
 *
 *  Return:	       void
 *
 *  Documentation: Callback-Funktion, die nur Hex-Werte zulaesst
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
void ModifyVerifyCheckOnHexCb(Widget w, XtPointer data, XtPointer cbs) {
  if(!ModifyVerifyCheckOnHex(w, (XmTextVerifyCallbackStruct *)cbs)) {
    ((XmTextVerifyCallbackStruct *)cbs)->doit = FALSE;
  }
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnOct
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs
 *
 *  Return:	       int
 *
 *  Documentation: Prueft, ob der Inhalt des TextFeldes eine Oktal-Zahl ist
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheckOnOct(Widget w, XmTextVerifyCallbackStruct *cbs) {
  return ModifyVerifyCheck(w, cbs, "01234567");
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnOctCb
 *  
 *  Input:         Widget w, XtPointer data, XtPointer cbs
 *
 *  Return:	       void
 *
 *  Documentation: Callback-Funktion, die nur Oktal-Werte zulaesst
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
void ModifyVerifyCheckOnOctCb(Widget w, XtPointer data, XtPointer cbs) {
  if(!ModifyVerifyCheckOnOct(w, (XmTextVerifyCallbackStruct *)cbs)) {
    ((XmTextVerifyCallbackStruct *)cbs)->doit = FALSE;
  }
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnBin
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs
 *
 *  Return:	       int
 *
 *  Documentation: Prueft, ob der Inhalt des TextFeldes eine Binaer-Zahl ist
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheckOnBin(Widget w, XmTextVerifyCallbackStruct *cbs) {
  return ModifyVerifyCheck(w, cbs, "01");
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnBinCb
 *  
 *  Input:         Widget w, XtPointer data, XtPointer cbs
 *
 *  Return:	       void
 *
 *  Documentation: Callback-Funktion, die nur Binaer-Werte zulaesst
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
void ModifyVerifyCheckOnBinCb(Widget w, XtPointer data, XtPointer cbs) {
  if(!ModifyVerifyCheckOnBin(w, (XmTextVerifyCallbackStruct *)cbs)) {
    ((XmTextVerifyCallbackStruct *)cbs)->doit = FALSE;
  }
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnLetter
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs
 *
 *  Return:	       int
 *
 *  Documentation: Prueft, ob der Inhalt des TextFeldes nur aus Buchstaben besteht
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheckOnLetter(Widget w, XmTextVerifyCallbackStruct *cbs) {
  return ModifyVerifyCheck(w, cbs, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnLetterCb
 *  
 *  Input:         Widget w, XtPointer data, XtPointer cbs
 *
 *  Return:	       void
 *
 *  Documentation: Callback-Funktion, die nur Buchstaben zulaesst
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
void ModifyVerifyCheckOnLetterCb(Widget w, XtPointer data, XtPointer cbs) {
  if(!ModifyVerifyCheckOnLetter(w, (XmTextVerifyCallbackStruct *)cbs)) {
    ((XmTextVerifyCallbackStruct *)cbs)->doit = FALSE;
  }
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnCName
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs
 *
 *  Return:	       int
 *
 *  Documentation: Prueft, ob der Inhalt des TextFeldes ein gueltiger C-Name ist
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheckOnCName(Widget w, XmTextVerifyCallbackStruct *cbs) {

  char *cname,c;

  if(!ModifyVerifyCheck(w, cbs, "_0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ")) {
    return FALSE;
  }
  cname = ModifyVerifyResult(w, cbs);
  if(!cname) return TRUE;
  if(!*cname) {XtFree(cname); return TRUE;}
  c = *cname;
  XtFree(cname);
  if(strchr("0123456789",c)) return FALSE;
  return TRUE;
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnCNameCb
 *  
 *  Input:         Widget w, XtPointer data, XtPointer cbs
 *
 *  Return:	       void
 *
 *  Documentation: Callback-Funktion, die nur C-Namen zulaesst
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
void ModifyVerifyCheckOnCNameCb(Widget w, XtPointer data, XtPointer cbs) {
  if(!ModifyVerifyCheckOnCName(w, (XmTextVerifyCallbackStruct *)cbs)) {
    ((XmTextVerifyCallbackStruct *)cbs)->doit = FALSE;
  }
}


/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyCheckOnLen
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs, int MaxLen
 *
 *  Return:	       int
 *
 *  Documentation: Prueft, ob im TextFeld maximal 'MaxLen' Zeichen stehen
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
int ModifyVerifyCheckOnLen(Widget w, XmTextVerifyCallbackStruct *cbs, int MaxLen) {

  char *text;
  int len;

  text = ModifyVerifyResult(w, cbs);
  if(!text) return TRUE;
  len = strlen(text);
  XtFree(text);
  return(len <= MaxLen);
}


/***************************************************************************
 *
 *  Funktionsname: ActivationHex2IntCb
 *  
 *  Input:         Widget w, XtPointer unused1, XtPointer unused2
 *
 *  Return:	       void
 *
 *  Documentation: Wandelt einen HexString in einen Integer um.
 *                 Zu Verwenden als Acticate-Callback
 *
 *
 *  History:       20.2.96 created by dn
 *
 *
 ***************************************************************************/
 
void ActivationHex2IntCb(Widget w, XtPointer unused1, XtPointer unused2) {

  long int erg=0;
  char *entry, buf[200];;

  entry = XmTextFieldGetString(w);
  if(entry) {
    erg = strtol(entry, NULL, 16);
    XtFree(entry);
  }
  sprintf(buf,"%ld",erg);
  XmTextFieldSetString(w,buf);
}

/***************************************************************************
 *
 *  Funktionsname: ModifyVerifyPasswordCb
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs, XtPointer data
 *
 *  Return:	   void 
 *
 *  Documentation: Used to display * instead of characters in a password 
 *                 textfield. The functions checks the length of the inserted
 *                 text. The new character is copied to the password buffer.
 *
 *
 *  History:       22.7.97 created by flux
 *
 *
 ***************************************************************************/
 
void ModifyVerifyPasswordCb(Widget w, XtPointer data, XmTextVerifyCallbackStruct *cbs) 
{
  if (cbs->text->ptr == NULL || cbs->text->length > 1) {
    cbs->doit = FALSE;
  } else {
    password = XtRealloc (password, sizeof (char) * text_index +2);
    password[text_index++] = cbs->text->ptr[0];
    password[text_index]   = '\0';

    cbs->text->ptr[0] = '*';
  }
}

/***************************************************************************
 *
 *  Funktionsname: MotionVerifyPasswordCb
 *  
 *  Input:         Widget w, XmTextVerifyCallbackStruct *cbs, XtPointer data
 *
 *  Return:	   void 
 *
 *  Documentation: The function is used to prevent the user from modifying text
 *                 after it has been entered.
 *
 *
 *  History:       22.7.97 created by flux
 *
 *
 ***************************************************************************/
void MotionVerifyPasswordCb(Widget w, XtPointer data, XmTextVerifyCallbackStruct *cbs)
{
  if (cbs->newInsert < cbs->currInsert)
      cbs->doit = FALSE;
}

/***************************************************************************
 *
 *  Funktionsname: MTextGetPasssword
 *  
 *  Input:         void
 *
 *  Return:	   char * // The password
 *
 *  Documentation: The function returns the string bufferd in 'password'.
 *                 The buffer is cleared and the text_index set to 0.
 *
 *
 *  History:       22.7.97 created by flux
 *
 *
 ***************************************************************************/
char * MTextGetPassword(void)
{
  char *tmp;

  if (password){
  	tmp = strdup(password);
	XtFree(password);
	text_index = 0;
   	return tmp;
  }
  return NULL;
}

